from __future__ import absolute_import


def test_main():
    from mixer.auto import mixer

    assert mixer


def test_django():
    from django.core.management import call_command
    from mixer.auto import mixer

    from .django_app.models import Rabbit

    call_command('migrate', interactive=False)

    rabbit = mixer.blend(Rabbit)
    assert rabbit

    rabbit = mixer.blend('tests.django_app.models.Rabbit')
    assert rabbit

    rabbits = mixer.cycle(2).blend(Rabbit)
    assert all(rabbits)

    call_command('flush', interactive=False)


def test_sqlalchemy():
    from mixer.auto import mixer
    from .test_sqlalchemy import User

    user = mixer.blend(User)
    assert user

    user = mixer.blend('tests.test_sqlalchemy.User')
    assert user

    users = mixer.cycle(2).blend(User)
    assert all(users)


def test_mongoengine():
    from mixer.backend.mongoengine import mixer as m
    m.params['commit'] = False

    from mixer.auto import mixer

    from .test_mongoengine import User

    user = mixer.blend(User)
    assert user

    user = mixer.blend('tests.test_mongoengine.User')
    assert user

    users = mixer.cycle(2).blend(User)
    assert all(users)



from __future__ import absolute_import
from mongoengine import *
import datetime


class User(Document):
    created_at = DateTimeField(default=datetime.datetime.now)
    email = EmailField(required=True)
    first_name = StringField(max_length=50)
    last_name = StringField(max_length=50)


class Comment(EmbeddedDocument):
    content = StringField()
    name = StringField(max_length=120)


class Post(Document):
    title = StringField(max_length=120, required=True)
    author = ReferenceField(User)
    category = StringField(choices=(
        ('S', 'Super'), ('M', 'Medium')), required=True)
    size = StringField(
        max_length=3, choices=('S', 'M', 'L', 'XL', 'XXL'), required=True)
    tags = ListField(StringField(max_length=30))
    comments = ListField(EmbeddedDocumentField(Comment))
    rating = DecimalField(precision=4, required=True)
    url = URLField(required=True)
    uuid = UUIDField(required=True)
    place = PointField()

    meta = {'allow_inheritance': True}


class Bookmark(Document):
    user = ReferenceField(User)
    bookmark = GenericReferenceField()


def test_generators():
    from mixer.backend.mongoengine import get_polygon

    polygon = get_polygon()
    assert polygon['coordinates']


def test_base():
    from mixer.backend.mongoengine import Mixer

    mixer = Mixer(commit=False)
    assert mixer

    now = datetime.datetime.now()

    user = mixer.blend(User)
    assert user.id
    assert user.email
    assert user.created_at
    assert user.created_at >= now


def test_typemixer():
    from mixer.backend.mongoengine import TypeMixer

    tm = TypeMixer(Post)
    post = tm.blend(comments=tm.RANDOM, place=tm.RANDOM)
    assert post.id
    assert post.title
    assert post.tags == []
    assert post.comments
    assert post.comments[0]
    assert isinstance(post.comments[0], Comment)
    assert post.author
    assert post.author.email
    assert post.rating
    assert post.category in ('S', 'M')
    assert '/' in post.url
    assert '-' in post.uuid
    assert 'coordinates' in post.place


def test_relation():
    from mixer.backend.mongoengine import Mixer

    mixer = Mixer(commit=False)

    post = mixer.blend(
        'tests.test_mongoengine.Post', author__username='foo')
    assert post.author.username == 'foo'

    bookmark = mixer.blend(Bookmark)
    assert not bookmark.bookmark

    bookmark = mixer.blend(Bookmark, bookmark=mixer.RANDOM)
    assert bookmark.bookmark


def test_embedded_document_list_field():
    from mixer.backend.mongoengine import Mixer

    class NewPost(Post):
        comments = EmbeddedDocumentListField(Comment, required=True)

    mixer = Mixer(commit=False)

    post = mixer.blend(NewPost)
    assert post.validate() is None
    assert len(post.comments) > 0


def test_decimal_field_min_value():
    from mixer.backend.mongoengine import Mixer

    class NewPost(Post):
        rating = DecimalField(required=True, min_value=0)

    mixer = Mixer(commit=False)

    post = mixer.blend(NewPost)
    assert post.rating > 0


def test_decimal_field_max_value():
    from mixer.backend.mongoengine import Mixer

    class NewPost(Post):
        rating = DecimalField(required=True, max_value=0)

    mixer = Mixer(commit=False)

    post = mixer.blend(NewPost)
    assert post.rating < 0


# pylama:ignore=W0401,W0614



from __future__ import absolute_import

from datetime import datetime

from flask import Flask
from flask_sqlalchemy import SQLAlchemy

db = SQLAlchemy()

usermessages = db.Table(
    'users_usermessages',
    db.Column('user_id', db.Integer, db.ForeignKey('user.id')),
    db.Column('message_id', db.Integer, db.ForeignKey('message.id'))
)


class Profile(db.Model):
    __tablename__ = 'profile'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(20), nullable=False)
    user = db.relationship("User", uselist=False, backref="profile")


class User(db.Model):

    __tablename__ = 'user'

    id = db.Column(db.Integer, primary_key=True)
    score = db.Column(db.SmallInteger, default=50, nullable=False)
    created_at = db.Column(db.DateTime, default=datetime.utcnow, nullable=False)
    username = db.Column(db.String(20), nullable=False)
    profile_id = db.Column(db.Integer, db.ForeignKey(
        'profile.id'), nullable=False)

    messages = db.relationship("Message", secondary=usermessages, backref="users")


class Role(db.Model):

    __tablename__ = 'role'

    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(20), nullable=False)
    user_id = db.Column(db.Integer, db.ForeignKey(User.id), nullable=False)

    user = db.relation(User)


class Message(db.Model):

    __tablename__ = 'message'

    id = db.Column(db.Integer, primary_key=True)
    content = db.Column(db.String(20))


class Node(db.Model):

    __tablename__ = 'node'

    id = db.Column(db.Integer, primary_key=True)
    parent_id = db.Column(db.Integer, db.ForeignKey('node.id'))
    children = db.relation(
        'Node',
        cascade='all',
        backref=db.backref('parent', remote_side='Node.id'))


app = Flask(__name__)
app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///:memory:'
db.init_app(app)


def test_base():
    from mixer.backend.flask import Mixer

    mixer = Mixer(commit=True)
    mixer.init_app(app)

    with app.test_request_context():
        db.create_all()

        node = mixer.blend('tests.test_flask.Node')
        assert node.id
        assert not node.parent

        role = mixer.blend('tests.test_flask.Role')
        assert role.user
        assert role.user_id == role.user.id

        user = mixer.blend(User)
        assert user.id
        assert user.username
        assert user.score == 50
        assert user.created_at
        assert user.profile
        assert user.profile.user == user

        user = mixer.blend(User, username='test')
        assert user.username == 'test'

        role = mixer.blend('tests.test_flask.Role', user__username='test2')
        assert role.user.username == 'test2'

        users = User.query.all()
        role = mixer.blend('tests.test_flask.Role', user=mixer.SELECT)
        assert role.user in users

        role = mixer.blend('tests.test_flask.Role', user=mixer.RANDOM)
        assert role.user

        profile = mixer.blend('tests.test_flask.Profile')
        user = mixer.blend(User, profile=profile)
        assert user.profile == profile

        user = mixer.blend(User, score=mixer.RANDOM)
        assert user.score != 50

        user = mixer.blend(User, username=lambda: 'callable_value')
        assert user.username == 'callable_value'

        # m2m
        messages = mixer.cycle(3).blend(Message)
        user = mixer.blend(User, messages=messages)
        assert len(user.messages) == 3

        user = mixer.blend(User, messages=mixer.RANDOM)
        assert len(user.messages) == 1

        user = mixer.blend(User, messages__content='message_content')
        assert len(user.messages) == 1
        assert user.messages[0].content == 'message_content'


def test_default_mixer():
    from mixer.backend.flask import mixer

    test = mixer.blend(User)
    assert test.username

# lint_ignore=F0401



from django.conf import settings
from django.contrib.contenttypes import models as ct_models

from django.db import models
from django.contrib.auth.models import User

from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation


class CustomField(models.CharField):

    """ Custom field. """

    pass


class Customer(User):
    name = models.CharField(max_length=100)


class Rabbit(models.Model):
    title = models.CharField(max_length=16)
    username = models.CharField(max_length=16, unique=True)
    active = models.BooleanField(default=True)
    email = models.EmailField()
    text = models.TextField(max_length=512)

    created_at = models.DateField()
    updated_at = models.DateTimeField()

    opened_at = models.TimeField()
    percent = models.FloatField()
    money = models.IntegerField()
    ip = models.IPAddressField()
    ip6 = models.GenericIPAddressField(protocol='ipv6')
    picture = models.FileField()

    some_field = models.CommaSeparatedIntegerField(max_length=12)
    funny = models.NullBooleanField(null=False, blank=False)
    slug = models.SlugField()
    speed = models.DecimalField(max_digits=3, decimal_places=1)

    url = models.URLField(null=True, blank=True, default='')

    file_path = models.FilePathField()
    content_type = models.ForeignKey(ct_models.ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    error_code = models.PositiveSmallIntegerField()
    custom = CustomField(max_length=24)
    content_object = GenericForeignKey('content_type', 'object_id')

    binary = models.BinaryField()

    one2one = models.OneToOneField('django_app.Simple', on_delete=models.CASCADE)

    def save(self, **kwargs):
        """ Custom save. """

        if not self.created_at:
            import datetime
            self.created_at = datetime.datetime.now()

        return super(Rabbit, self).save(**kwargs)


class Hole(models.Model):
    title = models.CharField(max_length=16)
    size = models.SmallIntegerField()
    owner = models.ForeignKey(Rabbit, on_delete=models.CASCADE)

    # FIXME compatibility
    rabbits = GenericRelation(Rabbit, **({'related_query_name': 'holes'}))


class Hat(models.Model):
    color = models.CharField(max_length=50, choices=(
        ('RD', 'red'),
        ('GRN', 'green'),
        ('BL', 'blue'),
    ))
    brend = models.CharField(max_length=10, default='wood')
    owner = models.ForeignKey(Rabbit, on_delete=models.CASCADE, null=True, blank=True)


class Silk(models.Model):
    color = models.CharField(max_length=20)
    hat = models.ForeignKey(Hat, on_delete=models.CASCADE)


class Door(models.Model):
    hole = models.ForeignKey(Hole, on_delete=models.CASCADE)
    owner = models.ForeignKey(Rabbit, on_delete=models.CASCADE, null=True, blank=True)
    size = models.PositiveIntegerField()


class Number(models.Model):
    doors = models.ManyToManyField(Door)
    wtf = models.ManyToManyField('self')


class ColorNumber(Number):
    color = models.CharField(max_length=20)


class Client(models.Model):
    username = models.CharField(max_length=20)
    city = models.CharField(max_length=20, unique=True)
    name = models.CharField(max_length=50)
    created_at = models.DateField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    score = models.IntegerField(default=50)


class Message(models.Model):
    content = models.TextField()
    client = models.ForeignKey(Client, on_delete=models.CASCADE)


class Tag(models.Model):
    title = models.CharField(max_length=20)

    customer = models.ForeignKey(Customer, on_delete=models.CASCADE, blank=True, null=True)
    messages = models.ManyToManyField(Message, null=True, blank=True)


class PointB(models.Model):
    pass


class PointA(models.Model):
    other = models.ManyToManyField("django_app.PointB",
                                   through="django_app.Through")


class Through(models.Model):
    pointas = models.ForeignKey(PointA, on_delete=models.CASCADE)
    pointbs = models.ForeignKey(PointB, on_delete=models.CASCADE)


class Simple(models.Model):
    value = models.IntegerField()



import pytest
import sys
from decimal import Decimal
from datetime import datetime


pytestmark = pytest.mark.skipif(
    sys.version_info >= (3, 10), reason='pony ORM doesnt support python 3.10')


def test_backend():
    from mixer.backend.pony import mixer
    assert mixer


def test_mixer():
    from pony import orm
    from mixer.backend.pony import mixer

    db = orm.Database("sqlite", ":memory:", create_db=True)

    class Customer(db.Entity):
        address = orm.Required(str)
        country = orm.Required(str)
        email = orm.Required(str, unique=True)
        name = orm.Required(str)
        password = orm.Required(str)

        cart_items = orm.Set("CartItem")
        orders = orm.Set("Order")

    class Product(db.Entity):
        id = orm.PrimaryKey(int, auto=True)
        name = orm.Required(str)
        categories = orm.Set("Category")
        description = orm.Optional(str)
        picture = orm.Optional(bytes)
        price = orm.Required(Decimal)
        quantity = orm.Required(int)
        cart_items = orm.Set("CartItem")
        order_items = orm.Set("OrderItem")

    class CartItem(db.Entity):
        quantity = orm.Required(int)
        customer = orm.Required(Customer)
        product = orm.Required(Product)

    class OrderItem(db.Entity):
        quantity = orm.Required(int, default=1)
        price = orm.Required(Decimal)
        order = orm.Required("Order")
        product = orm.Required(Product)
        orm.PrimaryKey(order, product)

    class Order(db.Entity):
        id = orm.PrimaryKey(int, auto=True)
        state = orm.Required(str)
        date_created = orm.Required(datetime)
        date_shipped = orm.Optional(datetime)
        date_delivered = orm.Optional(datetime)
        total_price = orm.Required(Decimal)
        customer = orm.Required(Customer)
        items = orm.Set(OrderItem)

    class Category(db.Entity):
        name = orm.Required(str, unique=True)
        products = orm.Set(Product)

    db.generate_mapping(create_tables=True)

    with orm.db_session:

        customer = mixer.blend(Customer)
        assert customer.name
        assert customer.email

        product = mixer.blend(Product)
        assert product.price

        order = mixer.blend(Order)
        assert order.customer

        orderitem = mixer.blend(OrderItem, product=product)
        assert orderitem.quantity == 1
        assert orderitem.order

        order = mixer.blend(Order, customer__name='John Snow')
        assert order.customer.name == 'John Snow'

        with mixer.ctx(commit=True):
            order = mixer.blend(Order)
            assert order.id



def test_faker():
    from mixer._faker import faker

    assert faker.big_integer()
    assert faker.ip_generic()

    assert faker.positive_decimal()
    assert faker.positive_decimal() > 0

    assert faker.positive_integer()
    assert faker.positive_integer() > 0

    assert faker.small_integer()

    assert faker.small_positive_integer()
    assert faker.small_positive_integer() > 0
    assert faker.small_positive_integer() <= 32767

    assert faker.uuid()

    assert 0 <= faker.percent() <= 100

    assert faker.percent_decimal()
    assert faker.title()
    assert faker.coordinates()

    env = faker.env

    name = faker.name
    faker.locale = 'ru'
    assert faker.day_of_week() in {
        "Воскресенье", "Понедельник", "Вторник", "Среда", "Четверг", "Пятница", "Суббота"}

    assert name()
    assert faker.name()

    faker.locale = 'en'
    assert faker.day_of_week() in {
        "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"}

    assert faker.name()
    assert faker.env is env

    assert faker.email()

    assert faker.pybytes()

    assert faker.date_time_this_month()



from __future__ import absolute_import

from datetime import datetime
from random import randrange

import pytest
from sqlalchemy import (
    Boolean,
    Column,
    DateTime,
    Enum,
    ForeignKey,
    Integer,
    SmallInteger,
    String,
    create_engine,
    types
)
from sqlalchemy.dialects import mssql, mysql, oracle, postgresql, sqlite
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relation, relationship, scoped_session, sessionmaker
from sqlalchemy.util import text_type

ENGINE = create_engine('sqlite:///:memory:')
BASE = declarative_base()
SESSION = scoped_session(sessionmaker(bind=ENGINE))


class Profile(BASE):
    __tablename__ = 'profile'

    id = Column(Integer, primary_key=True)
    name = Column(String(20), nullable=False)

    user = relationship("User", uselist=False, backref="profile")


class ProfileNonIncremental(BASE):
    __tablename__ = 'profile_nonincremental'

    id = Column(Integer, primary_key=True, autoincrement=False, nullable=False)
    name = Column(String(20), nullable=False)

    user = relationship("User", uselist=False, backref="profile_nonincremental")


class AugmentedType(types.TypeDecorator):
    impl = String


class User(BASE):
    __tablename__ = 'user'

    id = Column(Integer, primary_key=True)
    name = Column(String(10), nullable=False)
    role = Column(String(10), default='client', nullable=False)
    score = Column(SmallInteger, name='points', default=50, nullable=False)
    updated_at = Column(Boolean)
    created_at = Column(DateTime, default=datetime.utcnow, nullable=False)
    enum = Column(Enum('one', 'two'), nullable=False)
    random = Column(Integer, default=lambda: randrange(993, 995))
    profile_id = Column(Integer, ForeignKey('profile.id'), nullable=False)
    profile_id_nonincremental = Column(
        Integer, ForeignKey('profile_nonincremental.id'), nullable=False)
    augmented = Column(AugmentedType, default='augmented', nullable=False)


class Role(BASE):
    __tablename__ = 'role'

    name = Column(String(20), primary_key=True)
    user_id = Column(Integer, ForeignKey(User.id), nullable=False)

    user = relation(User)


BASE.metadata.create_all(ENGINE)


@pytest.fixture
def session():
    return SESSION()


def test_typemixer():
    from mixer.backend.sqlalchemy import TypeMixer

    mixer = TypeMixer(User)
    user = mixer.blend()
    assert user
    assert not user.id
    assert user.name
    assert 993 <= user.random < 995
    assert user.score == 50
    assert 2 < len(user.name) <= 10
    assert user.role == 'client'
    assert user.updated_at is None
    assert user.profile
    assert user.profile.user == user
    assert user.enum in ('one', 'two')
    assert user.augmented == 'augmented'

    user = mixer.blend(name='John', updated_at=mixer.RANDOM)
    assert user.name == 'John'
    assert user.updated_at in (True, False)

    mixer = TypeMixer('tests.test_sqlalchemy.Role')
    role = mixer.blend()
    assert role.user
    assert role.user_id == role.user.id


def test_mixer(session):
    from mixer.backend.sqlalchemy import Mixer

    mixer = Mixer(session=session, commit=True)
    p = mixer.blend('tests.test_sqlalchemy.ProfileNonIncremental', id=5)
    role = mixer.blend('tests.test_sqlalchemy.Role', user__profile_id_nonincremental=p)
    assert role and role.user

    role = mixer.blend(Role, user__name='test2')
    assert role.user.name == 'test2'

    profile = mixer.blend('tests.test_sqlalchemy.Profile')
    user = mixer.blend(User, profile__name='test')
    assert user.profile.name == 'test'

    user = mixer.blend(User, profile=profile)
    assert user.profile == profile

    user = mixer.blend(User, score=mixer.RANDOM)
    assert user.score != 50

    user = mixer.blend(User, username=lambda: 'callable_value')
    assert user.username == 'callable_value'


def test_cycle(session):
    from mixer.backend.sqlalchemy import Mixer

    mixer = Mixer(session=session, commit=True)
    profile1 = mixer.blend('tests.test_sqlalchemy.Profile', name='first')
    profile2 = mixer.blend('tests.test_sqlalchemy.Profile', name='second')
    users = mixer.cycle(2).blend(User, profile=(p for p in (profile1, profile2)))
    assert len(users) == 2
    assert users[0].profile.name == 'first'
    assert users[1].profile.name == 'second'


def test_select(session):
    from mixer.backend.sqlalchemy import Mixer

    mixer = Mixer(session=session, commit=True)

    users = session.query(User).all()
    role = mixer.blend(Role, user=mixer.SELECT)
    assert role.user in users

    user = users.pop()
    role = mixer.blend(Role, user=mixer.SELECT(User.id == user.id))
    assert user == role.user


def test_random():
    from mixer.backend.sqlalchemy import mixer

    values = ('mixer', 'is', 'fun')
    user = mixer.blend(User, name=mixer.RANDOM(*values))
    assert user.name in values


def test_default_mixer():
    from mixer.backend.sqlalchemy import mixer

    test = mixer.blend(User)
    assert test.name


def test_guard(session):
    from mixer.backend.sqlalchemy import Mixer, mixer

    with pytest.raises(ValueError):
        mixer.guard(User.name == 'maxi').blend(User)

    mixer = Mixer(session=session, commit=True)

    u1 = mixer.guard(User.name == 'maxi').blend(User, name='maxi')
    u2 = mixer.guard(User.name == 'maxi').blend(User)
    assert u1
    assert u1 == u2


def test_reload(session):
    from mixer.backend.sqlalchemy import Mixer

    mixer = Mixer(session=session, commit=True)

    u1 = mixer.blend(User)
    u1.name = 'wrong name'
    u2 = mixer.reload(u1)
    assert u2 == u1
    assert u2.name != 'wrong name'


def test_mix22(session):
    from mixer.backend.sqlalchemy import Mixer

    mixer = Mixer(session=session, commit=True)
    role = mixer.blend(Role, name=mixer.MIX.user.name)
    assert role.name == role.user.name


def test_nonincremental_primary_key(session):
    from mixer.backend.sqlalchemy import mixer

    test = mixer.blend(ProfileNonIncremental, id=42)
    assert test.name


def test_postgresql():
    from mixer.backend.sqlalchemy import TypeMixer
    from sqlalchemy.dialects.postgresql import UUID, JSONB

    base = declarative_base()

    class Test(base):
        __tablename__ = 'test'

        id = Column(Integer, primary_key=True)
        uuid = Column(UUID, nullable=False)
        jsonb = Column(JSONB, nullable=False)

    mixer = TypeMixer(Test)
    test = mixer.blend()
    assert test.uuid


@pytest.mark.parametrize('dialect, expected', [
    (mssql.dialect(), 'RAND()'),
    (mysql.dialect(), 'RAND()'),
    (oracle.dialect(), 'DBMS_RANDOM.VALUE'),
    (postgresql.dialect(), 'RANDOM()'),
    (sqlite.dialect(), 'RANDOM()'),
])
def test_random_compiled(dialect, expected):
    from mixer.backend.sqlalchemy import random
    compiled = random().compile(dialect=dialect)
    assert text_type(compiled) == expected
